using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace DinstantConsole
{
    public class Connection
    {
        public static int Port = 1800;
        private Socket socket;
        private Thread thread;
        private static TcpListener listener;
        private static Object lockerOn = new Object();
        //private static Object lockerOff = new Object();
        private Connection childConnection;
        private bool stop = false;
        private string endPoint;
        public Connection()
        {
            lock(lockerOn)
            {
                if(listener == null)
                {
                    listener = new TcpListener(IPAddress.Any, Port);
                    listener.Start();
                }
            }
        }

        public void Start()
        {
            thread = new Thread(() =>
            {
                try
                {
                    while(!listener.Pending())
                    {
                        if(stop)
                        {
                            return;
                        }
                        Thread.Sleep(100);
                    }
                    socket = listener.AcceptSocket();
                    childConnection = new Connection();
                    childConnection.Start();
                    var ep = socket.RemoteEndPoint as IPEndPoint;
                    endPoint = String.Format("{0}/{1}", ep.Address, ep.Port);
                    Console.ResetColor();
                    Console.WriteLine("Connected with "+endPoint);
                    StartReceive();
                }
                catch(Exception)
                {
                    //Console.WriteLine(ex.Message);
                }
            });
            thread.Start();

        }

        public void Stop()
        {
            stop = true;
            thread.Abort();
            if (socket != null)
            {
                socket.Close();
            }
            if (childConnection != null)
            {
                childConnection.Stop();
            }

            if(listener != null)
            {
                listener.Stop();
            }
        }

        private void StartReceive()
        {
            const int Buffersize = 1000;
            byte[] buffer = new byte[Buffersize];
            socket.BeginReceive(buffer, 0, Buffersize, SocketFlags.None, (state) =>
            {
                try
                {
                    int bytesReceived = socket.EndReceive(state);
                    for (int f = 0; f < bytesReceived; f++)
                    {
                        //Console.Write((char) buffer[f]);
                        ProcessChar((char)buffer[f]);
                    }
                    //Console.WriteLine(bytesReceived);

                    if (bytesReceived != 0)
                        StartReceive();
                    else
                        throw new Exception("got 0 data");
                }
                catch (Exception)
                {
                    //Console.WriteLine("Error..... " + e.Message);
                    Console.ResetColor();
                    Console.WriteLine("Disconnected with "+endPoint);
                }
            }, null);
        }

        private string ProcessBuffer = "";
        private int ProcessPayloadLength = -1;
        private bool FoundPipe1 = false;
        private bool FoundPipe2 = false;
        private void ProcessChar(char b)
        {
            try
            {
                if (!FoundPipe1)
                {
                    FoundPipe1 = b == '|';
                }
                else
                {
                    if (!FoundPipe2)
                    {
                        if (b == '|')
                        {
                            FoundPipe2 = true;
                        }
                        else
                        {
                            FoundPipe1 = false;
                        }
                    }
                    else
                    {
                        if (ProcessPayloadLength < 0)
                        {
                            ProcessBuffer += b;
                            if (ProcessBuffer.Length == 4)
                            {
                                ProcessPayloadLength = Convert.ToInt32(ProcessBuffer);
                                ProcessBuffer = "";
                                if (ProcessPayloadLength < 0)
                                {
                                    ProcessPayloadLength = 0;
                                }
                            }
                        }
                        else
                        {
                            ProcessBuffer += b;
                        }
                        if (ProcessPayloadLength == ProcessBuffer.Length)
                        {
                            WriteLine(ProcessBuffer);
                            ResetProcessBuffer();
                        }
                    }
                }
            }
            catch(Exception)
            {
                //Console.WriteLine(ex.Message);
                ResetProcessBuffer();
            }
        }

        static private Object writeLineLocker = new Object();
        static private void WriteLine(string aMessage)
        {
            lock(writeLineLocker)
            {
                if(aMessage.Length < 3)
                {
                    Console.ResetColor();
                    Console.WriteLine(aMessage);
                    return;
                }
                if(aMessage.Substring(0,2) == "$$")
                {
                    switch(aMessage[2])
                    {
                        case '1':
                            {
                                var _SList = new List<char>();
                                var _CList = new List<char>();
                                for(var i = 3; i < aMessage.Length; i++)
                                {
                                    switch(i%2)
                                    {
                                        case 0:
                                            _SList.Add(aMessage[i]);
                                            break;
                                        case 1:
                                            _CList.Add(aMessage[i]);
                                            break;
                                    }
                                }
                                for(var i = 0; i < _SList.Count; i++)
                                {
                                    Console.ForegroundColor = (ConsoleColor)(_CList[i]/16);
                                    Console.BackgroundColor = (ConsoleColor)(_CList[i]%16);
                                    if(_SList[i] == '\n')
                                    {
                                        while(Console.CursorLeft < Console.BufferWidth - 1)
                                        {
                                            Console.Write(' ');
                                        }
                                        Console.Write(' ');
                                    }
                                    else
                                    {
                                        Console.Write(_SList[i]);
                                    }
                                }
                                if (Console.CursorLeft > 0)
                                {
                                    while (Console.CursorLeft < Console.BufferWidth - 1)
                                    {
                                        Console.Write(' ');
                                    }
                                    Console.Write(' ');
                                }
                            }
                            break;
                        default:
                            Console.ResetColor();
                            Console.WriteLine(aMessage.Substring(3, aMessage.Length-3));
                            break;
                    }
                    return;
                }
                Console.ResetColor();
                Console.WriteLine(aMessage);
            }
        }

        private void ResetProcessBuffer()
        {
            FoundPipe1 = false;
            FoundPipe2 = false;
            ProcessPayloadLength = -1;
            ProcessBuffer = "";
        }
    }
}

